/* 
 * ****************************************************************************
 * RP6 ROBOT SYSTEM - ROBOT BASE EXAMPLES
 * ****************************************************************************
 * Example: SELFTEST PROGRAM
 * Author(s): Dominik S. Herwald
 * ****************************************************************************
 * Description:
 *
 * Test routines for all RP6 components. 
 * 
 * You can compile this in two different versions. The FACTORY Version is
 * more restrictive with the voltage sensor and tests if voltage is about 
 * 7.20V. 
 * You can not guarantee this with standard accumulators of course and thus
 * you would need a laboratory power supply - which most of the home users
 * will not have. 
 *
 * --------
 *
 * Yes I know - this program has not the most beautyful code on earth... 
 * To be honest it is quite ugly ;) 
 * All the Text output is just a waste of programspace and this is 
 * intentionally to get a large program ;) 
 *
 * It is a good demonstration how big your programs can get! (and this
 * program only fills 21KB of the 30KB)
 * Consider that every single character of a String consumes one byte 
 * of program space!
 * Also look at it in the hexfile viewer in RP6Loader - nearly half the 
 * program memory is full with ASCII Text.
 *  
 *
 * ****************************************************************************
 */
 
 
/*****************************************************************************/
// Includes:

#include "RP6RobotBaseLib.h" 	

/*****************************************************************************/

// Uncomment only ONE of the following definitions:
//#define FACTORY 
#define HOME

/*****************************************************************************/

char receiveBuffer[UART_RECEIVE_BUFFER_SIZE];

void done(void)
{
	writeString_P("Done!\n"); 
}

void bars(uint8_t number)
{
	uint8_t cnt;
	writeChar('\n');
	for(;number > 0; writeChar('\n'), number--)
		for(cnt = 69; cnt ; cnt--)
			writeChar('#');
}

void test(uint8_t number)
{
	bars(2);
	writeString_P("#### TEST #");
	writeInteger(number, DEC);
	writeString_P(" ####\n");
}

void printUBat(uint16_t uBat)
{
	writeIntegerLength((((uBat/102.4f)+0.1f)), DEC, 2);
	writeChar('.');
	writeIntegerLength((((uBat/1.024f)+10)), DEC, 2);
	writeChar('V');
}

/*****************************************************************************/
//

void testPowerOn(void)
{
	test(1);
	writeString_P("\n### POWER ON TEST ###\n");
	writeString_P("Please watch the yellow PowerOn LED and verify that it lights up!\n");
	writeString_P("(it will flash a few times!)\n");
	mSleep(1000);
	powerON();
	mSleep(1000);
	powerOFF();
	mSleep(1000);
	powerON();
	mSleep(2000);
	powerOFF();
	mSleep(500);
	powerON();
	mSleep(500);
	powerOFF();
	done();
}

/*****************************************************************************/
//

void testLEDs(void)
{
	test(2);
	writeString_P("\n### LED Test ###\n");
	writeString_P("Please watch the LEDs and verify that they all work!\n");
	mSleep(1000);
	setLEDs(0b111111);
	mSleep(500);
	setLEDs(0b000000);
	mSleep(500);
	setLEDs(0b111111);
	mSleep(500);
	setLEDs(0b000000);
	mSleep(250);
	uint8_t testLEDs = 1;
	uint8_t i;
	for(i = 0; i < 12; i++){
		setLEDs(testLEDs); 
		testLEDs <<= 1; 
		if(testLEDs > 63)
			testLEDs = 1; 	
		mSleep(400);
	}
	setLEDs(0b111111);
	mSleep(400);
	setLEDs(0b000000);
	done();
}

/*****************************************************************************/
//

void testVoltageSensor(void)
{
	test(3);
	writeString_P("\n### Voltage Sensor Test ###\n");
#ifdef FACTORY
	writeString_P("Please adjust the **laboratory power supply** to 7.20V!\n");
#endif
#ifdef HOME
	writeString_P("Be sure that you are using good accumulators!\n");
	writeString_P("\n### Enter \"x\" and hit return to START this test!\n\n");
    receiveBytesToBuffer(1, &receiveBuffer[0]);
#endif
	writeString_P("Performing 10 measurements:\n");
	uint16_t ubat;
	uint8_t i;
	for(i = 1; i <= 10; i++)
	{
		writeString_P("Measurement #"); 
		writeInteger(i, DEC);
		writeString_P(": ");
		ubat = readADC(ADC_BAT);
		printUBat(ubat);
#ifdef FACTORY
		if(ubat >= 700 && ubat <= 770)
		{
			writeString_P(" --> OK!\n");
		}
		else 
		{
			writeString_P("\n####### ERROR: VOLTAGE IS TOO ");
			if(ubat < 700) 
			{
				writeString_P("LOW");
			}
			else if(ubat > 770)
			{
				writeString_P("HIGH");
			}
			writeString_P(" (should be min. 6.90V to max. 7.60V)!\n");
		}
#endif
#ifdef HOME
		if(ubat >= 570 && ubat <= 970)
		{
			writeString_P(" --> OK!\n");
		}
		else 
		{
			writeString_P("\nWARNING: VOLTAGE IS TOO ");
			if(ubat < 570) 
			{
				writeString_P("LOW");
			}
			else if(ubat > 970)
			{
				writeString_P("HIGH");
			}
			writeString_P(" (should be min. 5.50V to max. 9.50V)!\n");
		}
#endif
		mSleep(200);
	}
	done();
}

/*****************************************************************************/
//

void testBumpers(void)
{
	test(4);
	writeString_P("\nBumper Test\n");
	writeString_P("Please hit both bumpers and verify \n");
	writeString_P("that both Bumpers are working properly!\n\n");
	writeString_P("### The Test is running now.\n### Enter \"x\" and hit return to STOP this test!\n\n");
	receiveBytes(1);
	while(true)
	{
		if(getUARTReceiveStatus() != UART_BUISY)
		{	
			copyReceivedBytesToBuffer(&receiveBuffer[0]);
			break;
		}
		
		static uint8_t info_left = false;
		static uint8_t info_right = false;
		uint8_t bumper_left = getBumperLeft();
		uint8_t bumper_right = getBumperRight();
		
		if(bumper_left) 
		{
			if(!info_left) 
			{			 
				writeString_P("OBSTACLE: LEFT!\n");
				info_left = true;
			}
		}
		else if(info_left) 
		{
			writeString_P("FREE: LEFT!\n");
			info_left = false;
		}
		if(bumper_right) 
		{
			if(!info_right) 
			{
				writeString_P("OBSTACLE: RIGHT!\n");
				info_right = true;
			}
		}
		else if(info_right) 
		{
			writeString_P("FREE: RIGHT!\n");
			info_right = false;
		}
	    statusLEDs.LED4 = (!bumper_left);
		statusLEDs.LED1 = (!bumper_right);
		updateStatusLEDs();
		mSleep(50);
	}
	setLEDs(0b000000);
	done();
}

/*****************************************************************************/
// 

void testLightSensors(void)
{
	test(5);
	writeString_P("\n### Light Sensor Test ###\n");
	writeString_P("Please get yourself a small flashlight!\n");
	writeString_P("While the test runs, move it in front of the Robot\n");
	writeString_P("and watch if the values change accordingly!\n\n");
#ifdef HOME
	writeString_P("### Enter \"x\" and hit return when you are ready!\n");
	receiveBytesToBuffer(1, &receiveBuffer[0]);
#endif
	writeString_P("### The Test is running now.\n### Enter \"x\" and hit return to STOP this test!\n\n");
	mSleep(1000);
	receiveBytes(1);
	startStopwatch1();
	startStopwatch2();
	writeString_P("Performing measurements...:\n");
	uint16_t sens_l;
	uint16_t sens_r;
	while(true)
	{
		if(getUARTReceiveStatus() != UART_BUISY)
		{	
			copyReceivedBytesToBuffer(&receiveBuffer[0]);
			break;
		}
		
		
		if(getStopwatch1() > 50) 
		{		
			sens_l = readADC(ADC_LS_L);
			sens_r = readADC(ADC_LS_R);
			
			int16_t dif = ((int16_t)(sens_l - sens_r));
			if(dif > 160) dif = 160;
			if(dif < -160) dif = -160;
	
			// Set LEDs - a small bargraph display that shows which sensor has detected
			// brighter light - and how much brighter it is:
			if(dif > 140)
				setLEDs(0b111000);
			else if(dif > 75)
				setLEDs(0b011000);
			else if(dif > 35)
				setLEDs(0b001000);
			else if(dif > -35)
				setLEDs(0b001001);
			else if(dif > -75)
				setLEDs(0b000001);
			else if(dif > -140)
				setLEDs(0b000011);
			else 
				setLEDs(0b000111);
			setStopwatch1(0);
		}
		
		if(getStopwatch2() > 250) 
		{	
			sens_l = readADC(ADC_LS_L);
			sens_r = readADC(ADC_LS_R);

			writeString_P("Left: ");
			writeIntegerLength(sens_l, DEC, 4);
			writeString_P(",  Right: ");
			writeIntegerLength(sens_r, DEC, 4);
			writeChar('\n');
			setStopwatch2(0);
		}
	}
	stopStopwatch1();
	stopStopwatch2();
	done();
} 

/*****************************************************************************/
//

uint8_t RC5_data;
uint8_t RC5_error;
uint8_t RC5_test;

void receiveRC5Data(RC5data_t rc5data)
{
	writeString_P("RX RC5 Packet --> Toggle Bit:");
	writeChar(rc5data.toggle_bit + '0');
	writeString_P(" | Device Address:");
	writeInteger(rc5data.device, DEC);
	writeString_P(" | Key Code:");
	writeInteger(rc5data.key_code, DEC);
	
	if(RC5_test==2)
	{
		writeString_P(" --> ");
		if(rc5data.key_code == RC5_data 
			&& rc5data.device == (RC5_data & 0b11111) 
			&& rc5data.toggle_bit == ((RC5_data>>5)&1))
		{
			writeString_P("OK!");
			RC5_error = false;
			RC5_test = 10;
		}
		else
		{
			writeString_P("### ERROR!!! ###");
			RC5_error = true;
			RC5_test = 5;
		}
	}
	writeChar('\n');
}


void testACS(void)
{
	test(6);
	writeString_P("\nACS Test\n");
	writeString_P("Please move your hand or other obstacles in front of the Robot\n");
	writeString_P("and verify that both ACS channels are working properly!\n\n");
	writeString_P("ACS is set to MAXIMUM power/range!\n\n");
	
	writeString_P("You can also send RC5 Codes with a TV Remote Control\n");
	writeString_P("to the RP6 - it will display the Toggle Bit, Device Adress\n");
	writeString_P("and Keycode of the RC5 Transmission!\n");
	writeString_P("Make sure your remote control transmits in RC5 and not\n");
	writeString_P("SIRCS or RECS80 etc.! There are several other formats that will NOT work!\n\n");
	writeString_P("### The Test is running now.\n### Enter \"x\" and hit return to STOP this test!\n\n");
	receiveBytes(1);
	
	powerON();
	setACSPwrHigh();

	while(true)
	{
		if(getUARTReceiveStatus() != UART_BUISY)
		{	
			copyReceivedBytesToBuffer(&receiveBuffer[0]);
			break;
		}
		task_ACS();

		static uint8_t info_left = false;
		static uint8_t info_right = false;
	
		if(isObstacleLeft()) 
		{
			if(!info_left) 
			{			 
				writeString_P("OBSTACLE: LEFT!\n");
				info_left = true;
			}
		}
		else if(info_left) 
		{
			writeString_P("FREE: LEFT!\n");
			info_left = false;
		}
		if(isObstacleRight()) 
		{
			if(!info_right) 
			{
				writeString_P("OBSTACLE: RIGHT!\n");
				info_right = true;
			}
		}
		else if(info_right) 
		{
			writeString_P("FREE: RIGHT!\n");
			info_right = false;
		}

		statusLEDs.byte = 0b000000;
		if(isObstacleLeft() && isObstacleRight())
			statusLEDs.byte = 0b100100;
		statusLEDs.LED5 = isObstacleLeft();
		statusLEDs.LED2 = isObstacleRight();
		statusLEDs.LED4 = (!isObstacleLeft());
		statusLEDs.LED1 = (!isObstacleRight());
		updateStatusLEDs();
	}
	setACSPwrOff();
	powerOFF();
	setLEDs(0b000000);
	done();
}

/*****************************************************************************/
//

void testRC5(void)
{
	test(7);
	writeString_P("\nIRCOMM Test\n");
	writeString_P("This test will transmit several RC5 Codes with the IRCOMM and check\n");
	writeString_P("if they are received back properly!\n");
	writeString_P("Please make sure that there is a white ceiling or some reflective\n");
	writeString_P("Object directly over the IRCOMM LEDs on the front of the RP6!\n\n");
	writeString_P("Please DO NOT send anything with TV Remote Controls or similar\n");
	writeString_P("as this will disturb the test and it will fail!\n\n");
	writeString_P("The ACS is active and the Robot will detect obstacles that you\nmove in front of it!\n\n");
#ifdef HOME
	writeString_P("\n### Enter \"x\" and hit return to START this test!\n\n");
	receiveBytesToBuffer(1, &receiveBuffer[0]);
#endif
	mSleep(500);
	powerON();
	setACSPwrOff();
	startStopwatch1();
	setLEDs(0b001000);
	RC5_data = 0;
	RC5_error = 0;
	RC5_test = 0;
	uint8_t errors = 0;
	uint8_t i = 0;
	while(true)
	{
		if(getStopwatch1() > 500) 
		{		
			if(RC5_test==2)
			{
				writeString_P("\nTIMEOUT!!!\n\n");
				errors = true;
				break;
			}
			writeString_P("TX RC5 Packet: ");
			writeInteger(i,DEC);
			writeChar('\n');
			
			IRCOMM_sendRC5(i, i);
			RC5_data = i;
			RC5_test = 2;
			i+=3;
			setStopwatch1(0);
		}
		if(RC5_test==10)
		{
			RC5_test = 0;
			setStopwatch1(450);
			if(i > 63)
			{
				writeString_P("\nTest finished!\n");
				break;
			}
		}
		if(RC5_test==5)
		{
			RC5_test = 0;
			writeString_P("\nRECEPTION ERROR!!!\n\n");
			errors = true;
			break;
		}
		
		task_ACS();
	}
	
	if(errors)
	{
		setLEDs(0b110110);
		writeString_P("####### !!!  WARNING WARNING WARNING  !!! #######\n");
		writeString_P("####### !!! TEST FINISHED WITH ERRORS !!! #######\n");
		writeString_P("####### !! Please check IRCOMM assembly ! #######\n");
		writeString_P("\n\nEnter any character to continue!\n");
		setACSPwrOff();
		powerOFF();
		receiveBytesToBuffer(1, &receiveBuffer[0]);
	}
	
	setACSPwrOff();
	stopStopwatch1();
	powerOFF();
	mSleep(1000);
	setLEDs(0b000000);
	done();
}

/*****************************************************************************/
//

void testMotorsAndEncoders(void)
{
	test(8);
	writeString_P("\nAutomatic speed speed regulation test\n\n");
	writeString_P("#####################################################################\n");
	writeString_P("### ATTENTION!!! DANGER!!! WARNING!!!\n");
	writeString_P("Make sure that the RP6 can __NOT__ move!\n");
	writeString_P("The caterpillar tracks should __NOT__ touch the ground!\n(hold it in your hands for example...)\n");
	writeString_P("THE RP6 WILL START MOVING FAST! YOU CAN DAMAGE IT IF YOU DO NOT\n");
	writeString_P("MAKE SURE THAT IT CAN __NOT__ MOVE!\n");
	writeString_P("Make sure both crawler tracks are FREE RUNNING! DO NOT BLOCK THEM!\n");
	writeString_P("--> OTHERWISE THE TEST WILL FAIL!\n");
	writeString_P("#####################################################################\n\n");
#ifdef HOME
	writeString_P("### Enter \"x\" and hit return when TO START this test!\n### Make sure the RP6 can not move!\n\n");
	receiveBytesToBuffer(1, &receiveBuffer[0]);
	if(receiveBuffer[0]!='x')
		return;
#endif
	task_RP6System();

	powerON();
	setMotorDir(FWD,FWD);
	setMotorPower(0,0);
	moveAtSpeed(0,0);
	setStopwatch1(0);
	setStopwatch2(0);
	startStopwatch1();
	startStopwatch2();
	uint8_t state = 0;
	uint8_t cnt = 0;
	uint8_t speed_test = 0;
	uint8_t errors = 0;
	task_RP6System();
	while(true)
	{
		if(getStopwatch2() > 400)
		{
			writeString_P("T: ");
			writeIntegerLength(speed_test,DEC,3);
			writeString_P(" |VL: ");
			writeIntegerLength(getLeftSpeed(),DEC,3);
			writeString_P(" |VR: ");
			writeIntegerLength(getRightSpeed(),DEC,3);
			writeString_P(" |PL: ");
			writeIntegerLength(getMotorLeft(),DEC,3);
			writeString_P(" |PR: ");
			writeIntegerLength(getMotorRight(),DEC,3);
			writeString_P(" |IL: ");
			writeIntegerLength(adcMotorCurrentLeft,DEC,3);
			writeString_P(" |IR: ");
			writeIntegerLength(adcMotorCurrentRight,DEC,3);
			writeString_P(" |UB: ");
			printUBat(adcBat);
			writeChar('\n');
			if((getLeftSpeed() < 130 && adcMotorCurrentLeft > 120)
			   || (getLeftSpeed() >= 130 && adcMotorCurrentLeft > 150))
			{
				writeString_P("####### Left: CURRENT CONSUMPTION IS TOO HIGH! ########\n");
				errors++;
			}
			if((getRightSpeed() < 130 && adcMotorCurrentRight > 120)
			   || (getRightSpeed() >= 130 && adcMotorCurrentRight > 150))
			{
				writeString_P("####### Right: CURRENT CONSUMPTION IS TOO HIGH! ########\n");
				errors++;
			}
			
			if(getLeftSpeed() > 25 && getMotorLeft() >= 30 && adcMotorCurrentLeft < 10)
			{
				writeString_P("####### Left: Current Sensor is defect!!! Current measurement is too low! ########\n");
				errors++;
			}
			if(getRightSpeed() > 25 && getMotorRight() >= 30 && adcMotorCurrentRight < 10)
			{
					writeString_P("####### Right: Current Sensor is defect!!! Current measurement is too low! ########\n");
				errors++;
			}
#ifdef HOME
			if(adcBat < 550)
			{
				writeString_P("####### ERROR: Supply Voltage is too low (<5.50V)! Please check power supply! #######\n");
				errors++;
			}
#endif
#ifdef FACTORY
			if(adcBat < 650)
			{
				writeString_P("####### ERROR: Supply Voltage is too low (<6.50V)! Please check power supply! #######\n");
				errors++;
			}
#endif
			if(errors)
				break;

			setStopwatch2(0);
		}
		if(getStopwatch1() > 1000)
		{
				moveAtSpeed(speed_test,speed_test);
				if(cnt > 2)
				{
					if((getLeftSpeed() >= ((speed_test > 5) ? (speed_test - 10) : speed_test)) 
						&& (getLeftSpeed() <= (speed_test+10)))
					{
						writeString_P("Speed Left: OK\n");
					}
					else
					{
						writeString_P("####### ERROR Left ########\n");
						errors++;
					}

					if((getRightSpeed() >= ((speed_test > 5) ? (speed_test - 10) : speed_test)) 
						&& (getRightSpeed() <= (speed_test+10)))
					{
						writeString_P("Speed Right: OK\n");
					}
					else
					{
						writeString_P("####### ERROR Right #######\n");
						errors++;
					}
#ifdef FACTORY
					if(getMotorLeft() > (3*getLeftSpeed()))
					{
						writeString_P("####### ERROR: LEFT PWM Value is too high! Check motor assembly! #######\n");
						errors++;
						break;
					}
					if(getMotorRight() > (3*getRightSpeed()))
					{
						writeString_P("####### ERROR: LEFT PWM Value is too high! Check motor assembly! #######\n");
						errors++;
						break;
					}
#endif
					if(errors) break;

					if(state == 1)
					{
						if(speed_test <= 120)
						{
							if(speed_test >= 80)
								changeDirection(BWD);
							else if(speed_test >= 60)
								changeDirection(FWD);
							else if(speed_test >= 40)
								changeDirection(BWD);
							else if(speed_test < 40)
								state = 2;
						}
						speed_test -= 20;
#ifdef FACTORY
						if(speed_test >= 120)
							speed_test -= 20;
#endif
					}

					if(state == 0)
					{
#ifdef FACTORY
						if(speed_test >= 160)
							state = 1;
						if(speed_test >= 80)
							speed_test += 20;
#endif
#ifdef HOME
						if(speed_test >= 80)
							state = 1;
#endif
						speed_test += 20;
					}
					cnt = 0;
				}
				else
					cnt++;
			setStopwatch1(0);
		}
		task_RP6System();

		if(state==2)
		{
			moveAtSpeed(0,0);
			setMotorPower(0,0);
			break;
		}
		task_RP6System();
	}

	stopStopwatch1();
	setMotorPower(0,0);
	moveAtSpeed(0,0);
	setMotorDir(FWD,FWD);
	powerOFF();
	if(errors)
	{
		setLEDs(0b110110);
		writeString_P("####### !!!  WARNING WARNING WARNING  !!! #######\n");
		writeString_P("####### !!! TEST FINISHED WITH ERRORS !!! #######\n");
		writeString_P("#### Please check motor and encoder assembly! ###\n");
		writeString_P("#### And also check Sensors for defects!      ###\n");
		writeString_P("\n\nEnter any character to continue!\n");
		powerOFF();
		receiveBytesToBuffer(1, &receiveBuffer[0]);
	}
	else
	{
		writeString_P("\n***** MOTOR AND ENCODER TEST OK! *****\n");
		mSleep(1000);
	}
	setLEDs(0b000000);
}

void printMotorValues(void)
{
	writeString_P("PL: ");
	writeIntegerLength(getMotorLeft(),DEC,3);
	writeString_P(" | PR: "); 
	writeIntegerLength(getMotorRight(),DEC,3);
	writeString_P(" | IL: ");
	writeIntegerLength(adcMotorCurrentLeft,DEC,3);
	writeString_P(" | IR: ");
	writeIntegerLength(adcMotorCurrentRight,DEC,3);
	writeString_P(" | VL: ");
	writeIntegerLength(getLeftSpeed(),DEC,3);
	writeString_P(" | VR: ");
	writeIntegerLength(getRightSpeed(),DEC,3);
	writeChar('\n');
	writeString_P(" | DSTR: ");
	writeInteger(getRightDistance(),DEC);
	writeString_P(" | DSTL: ");
	writeInteger(getLeftDistance(),DEC);
	writeString_P(" ###  | UBAT: "); 
	printUBat(adcBat);
	writeChar('\n');
}

void speed_control(void)
{
	bars(2);
	writeString_P("\nManual speed regulation test\n\n");
	writeString_P("#####################################################################\n");
	writeString_P("### ATTENTION!!! DANGER!!! WARNING!!!\n");
	writeString_P("Make sure that the RP6 can __NOT__ move!\n");
	writeString_P("The caterpillar tracks should __NOT__ touch the ground!\n(hold it in your hands for example...)\n");
	writeString_P("THE RP6 CAN START MOVING VERY FAST! YOU CAN DAMAGE IT IF YOU DO NOT\n");
	writeString_P("MAKE SURE THAT IT CAN __NOT__ MOVE!\n");
	writeString_P("#####################################################################\n\n");
	writeString_P("\n\nUsage: Enter Speed values in the following format: 000 \n");
	writeString_P("More examples: 010, 025, 068, 094, 100, 120, 155, 200 (max!)\n");
	writeString_P("Enter \"end\" and hit return when you want to stop the test!\n");
	writeString_P("Enter \"x\" and hit return when you are ready!\n");
	receiveBytesToBuffer(1, &receiveBuffer[0]);
	if(receiveBuffer[0]!='x')
		return;
	receiveBuffer[0]=0;
	receiveBytes(3);
	int8_t i = 0;

	powerON();
	setMotorDir(FWD,FWD);
	setMotorPower(0,0);
	moveAtSpeed(0,0);
	uint8_t speed_value = 0;
	uint8_t speed_value_tmp = 0;
	uint8_t digit = 1;
	startStopwatch1();
	while(true)
	{
		if(getUARTReceiveStatus() != UART_BUISY)
		{	
			copyReceivedBytesToBuffer(&receiveBuffer[0]);
			if(receiveBuffer[0]=='e' && receiveBuffer[1]=='n' && receiveBuffer[2]=='d' )
			{
			    writeString_P("\nEND\n");
				break;
			}
			else if(receiveBuffer[0]=='f' && receiveBuffer[1]=='w' && receiveBuffer[2]=='d' )
			{
			    writeString_P("\nFWD\n");
				changeDirection(FWD);
				receiveBytes(3);
			}
			else if(receiveBuffer[0]=='b' && receiveBuffer[1]=='w' && receiveBuffer[2]=='d' )
			{
			    writeString_P("\nBWD\n");
				changeDirection(BWD);
				receiveBytes(3);
			}
			else
			{
				digit = 1;
				for(i = 2 ; i >= 0; i--)
				{
					if((receiveBuffer[i] >= 48) && (receiveBuffer[i] <= 58))
					{
						speed_value_tmp += (receiveBuffer[i]-48) * digit;
						digit *= 10;
						if(i == 0) 
						{
							speed_value = speed_value_tmp;
							speed_value_tmp = 0;
							moveAtSpeed(speed_value,speed_value);
							break;
						}
					}
					else
					{
						speed_value_tmp = 0;
						break;
					}
				}
				receiveBytes(3);
			}
		}
		if(getStopwatch1() > 250)
		{
			printMotorValues();
			setStopwatch1(0);
		}
		task_RP6System();
	}
	stopStopwatch1();
	mSleep(500);
	setMotorPower(0,0);
	moveAtSpeed(0,0);
	setMotorDir(FWD,FWD);
	powerOFF();
	done();
}

void testMoveDistance(void)
{
	bars(2);
	writeString_P("\nMove Distance test\n");
	writeString_P("#####################################################################\n");
	writeString_P("### ATTENTION!!! DANGER!!! WARNING!!!\n");
	writeString_P("Make sure that the RP6 can __NOT__ move!\n");
	writeString_P("The caterpillar tracks should __NOT__ touch the ground!\n(hold it in your hands for example...)\n");
	writeString_P("THE RP6 CAN START MOVING VERY FAST! YOU CAN DAMAGE IT IF YOU DO NOT\n");
	writeString_P("MAKE SURE THAT IT CAN __NOT__ MOVE!\n");
	writeString_P("#####################################################################\n\n");
	writeString_P("\n\nUsage: Enter Distance values in the following format: 000 \n");
	writeString_P("More examples: 150, 250, 200, 400, 555, 999 (max!)\n");
	writeString_P("Enter \"end\" and hit return when you want to stop the test!\n");
	writeString_P("Enter \"x\" and hit return when you are ready!\n");
	receiveBytesToBuffer(1, &receiveBuffer[0]);
	if(receiveBuffer[0]!='x')
		return;
	receiveBuffer[0]=0;
	receiveBytes(3);
	int8_t i = 0;

	powerON();
	setMotorDir(FWD,FWD);
	setMotorPower(0,0);
	moveAtSpeed(0,0);
	startStopwatch1();
	receiveBytes(3);
	uint16_t dist = 0;
	uint16_t dist_tmp = 0;
	uint8_t digit = 0;
	while(true)
	{
		if(getUARTReceiveStatus() != UART_BUISY)
		{	
			copyReceivedBytesToBuffer(&receiveBuffer[0]);
			if(receiveBuffer[0]=='e' && receiveBuffer[1]=='n' && receiveBuffer[2]=='d' )
			{
			    writeString_P("\nEND\n");
				break;
			}
			else if(receiveBuffer[0]=='f' && receiveBuffer[1]=='w' && receiveBuffer[2]=='d' )
			{
			    writeString_P("\nFWD\n");
				changeDirection(FWD);
				receiveBytes(3);
			}
			else if(receiveBuffer[0]=='b' && receiveBuffer[1]=='w' && receiveBuffer[2]=='d' )
			{
			    writeString_P("\nBWD\n");
				changeDirection(BWD);
				receiveBytes(3);
			}
			else
			{
				digit = 1;
				for(i = 2 ; i >= 0; i--)
				{
					if((receiveBuffer[i] >= 48) && (receiveBuffer[i] <= 58))
					{
						dist_tmp += (receiveBuffer[i]-48) * digit;
						digit *= 10;
						if(i == 0) 
						{
							dist = dist_tmp*10;
     						move(40,FWD,dist,false);
							dist_tmp = 0;
							break;
						}
					}
					else
					{
						dist_tmp = 0;
						break;
					}
				}
				receiveBytes(3);
			}
		}
		if(getStopwatch1() > 250)
		{
			printMotorValues();
			setStopwatch1(0);
		}
		task_RP6System();
	}
	stopStopwatch1();
	mSleep(500);
	setMotorPower(0,0);
	moveAtSpeed(0,0);
	setMotorDir(FWD,FWD);
	powerOFF();
	done();
}


void testEncoderDutyCycle(void)
{
	bars(2);
	writeString_P("\nEncoder Duty Cycle test\n\n");
	writeString_P("#####################################################################\n");
	writeString_P("### ATTENTION!!! DANGER!!! WARNING!!!\n");
	writeString_P("Make sure that the RP6 can __NOT__ move!\n");
	writeString_P("The caterpillar tracks should __NOT__ touch the ground!\n(hold it in your hands for example...)\n");
	writeString_P("THE RP6 CAN START MOVING VERY FAST! YOU CAN DAMAGE IT IF YOU DO NOT\n");
	writeString_P("MAKE SURE THAT IT CAN __NOT__ MOVE!\n");
	writeString_P("#####################################################################\n\n");
	
	writeString_P("Speed regulation is disabled - the motors just get constant\n");
	writeString_P("Power - this allows you to adjust the small potis on the Encoders\n");
	writeString_P("to set the duty cycle properly.\n");
	writeString_P("Optimal is a value of 50% - but if it jumps up and down between\n");
	writeString_P("20% and 80% this is also OK.\n");
	writeString_P("A few \"BAD!!!\" messages (maybe 2 each 5 seconds) are normal and\n");
	writeString_P("nothing to worry about.\n");
	writeString_P("If there are more error messages - you need to adjust the potis a bit\n");
	writeString_P("more to make it work!\n");
	writeString_P("\nEnter \"x\" and hit return when you are ready!\n");
	receiveBytesToBuffer(1, &receiveBuffer[0]);
	if(receiveBuffer[0]!='x')
		return;
	receiveBuffer[0]=0;
	receiveBytes(3);

	powerON();
	setMotorDir(FWD,FWD);
	moveAtSpeed(0,0);
	startStopwatch1();
	setMotorPower(55,55);
	mSleep(500);
			receiveBytes(1);
	while(true)
	{
		if(getUARTReceiveStatus() != UART_BUISY)
		{	
			copyReceivedBytesToBuffer(&receiveBuffer[0]);
			break;
		}
	
		if(getStopwatch1() > 200)
		{
			if(getLeftSpeed() < 10)
			{
				writeString_P("SPEED LEFT IS TOO LOW!!!!\n");
			}
			if(getRightSpeed() < 10)
			{
				writeString_P("SPEED RIGHT IS TOO LOW!!!!\n");
			}
			
			uint16_t cycle = cycle_h_l + cycle_l_l;
			uint8_t dc_high =(uint8_t) ((cycle_h_l*100) / cycle);
			writeString_P("Duty Cycle + LEFT: ");
			writeInteger(dc_high,DEC);
			writeChar('%');
			if(dc_high >= 75 || dc_high < 30)
			{
				writeString_P(" -> BAD !!!");
			}
			else if(dc_high >= 30)
			{
				writeString_P(" -> OK");
			}
		
			cycle = cycle_h_r + cycle_l_r;
			dc_high = (uint8_t) ((cycle_h_r*100) / cycle);
			
			writeString_P(" || Duty Cycle + RIGHT: ");
			writeInteger(dc_high,DEC);
			writeChar('%');
			if(dc_high >= 75 || dc_high < 30)
			{
				writeString_P(" BAD !");
			}
			else if(dc_high >= 30)
			{
				writeString_P(" OK");
			}
			
			writeString_P("  | VL:"); 
			writeInteger(getLeftSpeed(),DEC);
			writeString_P("  | VR:"); 
			writeInteger(getRightSpeed(),DEC);
			writeChar('\n');
			setStopwatch1(0);
		}
	}
	stopStopwatch1();
	mSleep(500);
	setMotorPower(0,0);
	setMotorDir(FWD,FWD);
	powerOFF();
	done();
}



/*****************************************************************************/
// Main:

int main(void)
{
	initRobotBase(); 
	
	uint8_t test = 0;

	setLEDs(0b111111);
	mSleep(500);
	setLEDs(0b000000);
	mSleep(250);
	
	IRCOMM_setRC5DataReadyHandler(receiveRC5Data);
	writeString_P("\n\nRP6 Robot Base Selftest\n\n");
#ifdef FACTORY
	writeString_P("\n\nATTENTION: FACTORY VERSION! \n"); 
	writeString_P("You need to use a laboratory Power supply set to 7.20V!\n\n"); 
#endif
	writeString_P("#####################################################################\n");
	writeString_P("### ATTENTION!!! DANGER!!! WARNING!!!\n");
	writeString_P("Make sure that the RP6 CAN __NOT__ MOVE!\n");
	writeString_P("The caterpillar tracks should NOT touch the ground!\n(hold it in your hands for example...)\n");
	writeString_P("THE RP6 WILL START MOVING IN TEST 8! YOU CAN DAMAGE IT IF YOU DO NOT\n");
	writeString_P("MAKE SURE THAT IT CAN __NOT__ MOVE!\n");
	writeString_P("Make sure both crawler tracks are FREE RUNNING! DO NOT BLOCK THEM!\n");
	writeString_P("#####################################################################\n\n");
	writeString_P("Please enter \"x\" to continue!\n"); 
	receiveBytesToBuffer(1, &receiveBuffer[0]);

	
	while(true)
	{
		// This menu is mainly a program space filler - good for serial interface (speed) testing ;)
		// The same applies for ALL other text information in this program!
		// All this is just to have a BIG program download ;)
		writeChar('\n');
		writeString_P("#####################################################################\n"); 
		writeString_P("#########            RP6 Robot Base Selftest                #########\n"); 
#ifdef FACTORY
		writeString_P("######### FACTORY VERSION (!! 7.2V !!)  v. 1.1 - 14.06.2007 #########\n"); 
#endif
#ifdef HOME
		writeString_P("######### HOME VERSION                  v. 1.1 - 14.06.2007 #########\n"); 
#endif
	    writeString_P("#####################################################################\n"); 
		writeString_P("#####       Main Menu         #########      Advanced Menu      #####\n"); 
		writeString_P("#                                 #                                 #\n"); 
		writeString_P("# 0 - Run ALL Selftests (0-8)     # s - Move at speed Test          #\n"); 
		writeString_P("# 1 - PowerOn Test                # d - Move distance Test          #\n"); 
		writeString_P("# 2 - LED Test                    # c - Encoder Duty-Cycle Test     #\n"); 
		writeString_P("# 3 - Voltage Sensor Test         #                                 #\n"); 
		writeString_P("# 4 - Bumper Test                 #                                 #\n"); 
		writeString_P("# 5 - Light Sensor Test           #                                 #\n"); 
		writeString_P("# 6 - ACS (and RC5 receive) Test  #                                 #\n"); 
		writeString_P("# 7 - IRCOMM/RC5 Test             # System voltage is:              #\n"); 
		writeString_P("# 8 - Motors and Encoders Test    #      "); 
		printUBat(readADC(ADC_BAT));
		writeString_P("                     #\n"); 
		writeString_P("#                                 #                                 #\n"); 
		writeString_P("#####################################################################\n"); 
		writeString_P("# Please enter your choice (1-8, s, d, c)!                          #\n"); 
		writeString_P("#####################################################################\n"); 


		receiveBytesToBuffer(1, &receiveBuffer[0]);
	
		test = receiveBuffer[0] - 48;
		
		if(receiveBuffer[0] == 's')
		{
			// The SpeedControl test...
			speed_control();
		}
		else if(receiveBuffer[0] == 'd')
		{
			testMoveDistance();
		}
		else if(receiveBuffer[0] == 'c')
		{
			testEncoderDutyCycle();
		}
		else if(test > 8)
		{
			writeString_P("You need to enter a single number from 0 to 8, s, d or a!");
			continue;
		}
		else
		{
			switch(test)
			{
				case 0: 
					testPowerOn();
					testLEDs();
					testVoltageSensor(); 
					testBumpers(); 
					testLightSensors(); 
					testACS(); 
					testRC5(); 
					testMotorsAndEncoders();
				break; 	
				case 1: testPowerOn(); 				break; 
				case 2: testLEDs(); 				break; 
				case 3: testVoltageSensor();		break; 		
				case 4: testBumpers(); 				break;	
				case 5: testLightSensors(); 		break;
				case 6: testACS(); 			 		break;
				case 7: testRC5(); 		 	 		break;
				case 8: testMotorsAndEncoders(); 	break;	
			}
		}
	}
	return 0;
}


